package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.busi.jx;

import cn.estsh.i3plus.ext.mes.api.busi.jx.IJxCenterWorkingBoardService;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesOutPutStatistics;
import cn.estsh.i3plus.ext.mes.pojo.bean.MesWorkCenterExt;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @Author : wangjie
 * @CreateDate : 2022-02-15
 * @Modify:
 **/
@Slf4j
public class JxCenterWorkingBoardBusiThreeService extends JxCenterWorkingBoardCommonService implements IJxCenterWorkingBoardService {

    private IJxCenterWorkingBoardService proxyService;

    public JxCenterWorkingBoardBusiThreeService(IJxCenterWorkingBoardService proxyService) {
        this.proxyService = proxyService;
    }

    @Override
    public Map<String, Object> queryCenterWorkingBoardData(Map<String, Object> dataMap) {

        dataMap = proxyService.queryCenterWorkingBoardData(dataMap);

        MesWorkCenterExt workCenterExtDb = (MesWorkCenterExt) dataMap.get(MesWorkCenterExt.class.getSimpleName());

        List<MesWorkOrder> workOrderList = (List<MesWorkOrder>) dataMap.get(MesWorkOrder.class.getSimpleName());

        List<MesOutPutStatistics> outPutStatisticsList = (List<MesOutPutStatistics>) dataMap.get(MesOutPutStatistics.class.getSimpleName());

        Optional<MesWorkOrder> curWorkOrder = getFirstWorkOrder(workOrderList, outPutStatisticsList, workCenterExtDb);

        List<MesOutPutStatistics> curOutPutStatisticsList = getFirstOutPutStatisticsList(outPutStatisticsList, curWorkOrder);

        dataMap.put("当前生产机种名", getCurProdPartNo(curOutPutStatisticsList, curWorkOrder));

        dataMap.put("当前计划台数", getQtyStr(getCurPlanQty(curOutPutStatisticsList, curWorkOrder, workCenterExtDb, dataMap)));

        dataMap.put("当前实绩台数", getQtyStr(dataMap.containsKey(MesExtConstWords.COMPLET_QTY) ? (Double) dataMap.get(MesExtConstWords.COMPLET_QTY) : getOutPutStatisticsList2AmountQty(curOutPutStatisticsList)));

        return clearUselessData(dataMap);
    }

    @Override
    public Map<String, Object> clearUselessData(Map<String, Object> dataMap) {
        dataMap.remove(MesWorkCenterExt.class.getSimpleName());
        dataMap.remove(MesWorkOrder.class.getSimpleName());
        dataMap.remove(MesOutPutStatistics.class.getSimpleName());
        dataMap.remove(MesExtConstWords.COMPLET_QTY);
        return dataMap;
    }

    private Optional<MesWorkOrder> getFirstWorkOrder(List<MesWorkOrder> workOrderList, List<MesOutPutStatistics> outPutStatisticsList, MesWorkCenterExt workCenterExtDb) {

        if (!CollectionUtils.isEmpty(outPutStatisticsList)) return CollectionUtils.isEmpty(workOrderList) ? null : workOrderList.stream().filter(o -> (null != o && o.getOrderNo().equals(outPutStatisticsList.get(0).getWorkOrderNo()))).findFirst();

        List<MesWorkOrder> workOrderList2Sort = getWorkOrderList2Sort(workOrderList, outPutStatisticsList, workCenterExtDb);

        return CollectionUtils.isEmpty(workOrderList2Sort) ? null : Optional.of(workOrderList2Sort.get(0));

    }

    private List<MesOutPutStatistics> getFirstOutPutStatisticsList(List<MesOutPutStatistics> outPutStatisticsList, Optional<MesWorkOrder> curWorkOrder) {
        String curWorkOrderNo = (null != curWorkOrder && curWorkOrder.isPresent()) ? curWorkOrder.get().getOrderNo() : (!CollectionUtils.isEmpty(outPutStatisticsList) ? outPutStatisticsList.get(0).getWorkOrderNo() : null);
        return (StringUtils.isEmpty(curWorkOrderNo) || CollectionUtils.isEmpty(outPutStatisticsList)) ? null : outPutStatisticsList.stream().filter(o -> (null != o && o.getWorkOrderNo().equals(curWorkOrderNo))).collect(Collectors.toList());
    }

    private Object getCurProdPartNo(List<MesOutPutStatistics> curOutPutStatisticsList, Optional<MesWorkOrder> curWorkOrder) {
        return !CollectionUtils.isEmpty(curOutPutStatisticsList) ? curOutPutStatisticsList.get(0).getPartNo() : ((null != curWorkOrder && curWorkOrder.isPresent()) ? curWorkOrder.get().getPartNo() : MesExtConstWords.EMPTY);
    }

    private Double getCurPlanQty(List<MesOutPutStatistics> curOutPutStatisticsList, Optional<MesWorkOrder> curWorkOrder, MesWorkCenterExt workCenterExtDb, Map<String, Object> dataMap) {
        return (null == curWorkOrder || !curWorkOrder.isPresent()) ? MesExtConstWords.ZERO :
                (curWorkOrder.get().getStartTime().contains(workCenterExtDb.getWorkTime()) ? curWorkOrder.get().getQty() :
                        MathOperation.add(MathOperation.sub(curWorkOrder.get().getQty(), curWorkOrder.get().getCompleteQty()), amountOutPutStatisticsQty(curOutPutStatisticsList, dataMap)));
    }

    private Double amountOutPutStatisticsQty(List<MesOutPutStatistics> outPutStatisticsList, Map<String, Object> dataMap) {
        Double qty = getOutPutStatisticsList2AmountQty(outPutStatisticsList);
        dataMap.put(MesExtConstWords.COMPLET_QTY, qty);
        return qty;
    }

}
